// File created from ../../../src/lib/log/logimpl_messages.mes

#ifndef LOGIMPL_MESSAGES_H
#define LOGIMPL_MESSAGES_H

#include <log/message_types.h>

namespace isc {
namespace log {

extern const isc::log::MessageID LOGIMPL_ABOVE_MAX_DEBUG;
extern const isc::log::MessageID LOGIMPL_BAD_DEBUG_STRING;
extern const isc::log::MessageID LOGIMPL_BELOW_MIN_DEBUG;

} // namespace log
} // namespace isc

#endif // LOGIMPL_MESSAGES_H
